/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    rl_net_lib.h
 * Purpose: Network Library Definitions
 * Rev.:    V7.1.0
 *----------------------------------------------------------------------------*/

#ifndef __RL_NET_LIB_H
#define __RL_NET_LIB_H

#include "Driver_USART.h"
#include "Driver_ETH_MAC.h"
#include "Driver_ETH_PHY.h"
#include "net_modem.h"
#include "rl_net_ds.h"

/// General definitions
#define NET_HEADER_LEN      4           ///< Network frame header length
#define PHY_HEADER_LEN      14          ///< Physical/ethernet header length
#define ETH_MAX_FRAME_SIZE  1514        ///< Maximum size of ethernet frame
#define NET_USERNAME_SIZE   16          ///< Authentication username buffer size
#define NET_PASSWORD_SIZE   16          ///< Authentication password buffer size
#define NET_ROOT_PATH_SIZE  128         ///< Root path buffer size
#define NET_TCP_NO_ACK      0xFFFFFFFF  ///< Callback return value for no ACK

/// Network Interfaces
#define NETIF_ETH           0           ///< Ethernet
#define NETIF_PPP           1           ///< PPP
#define NETIF_SLIP          2           ///< SLIP
#define NETIF_LOCAL         3           ///< Localhost (loopback)
#define NETIF_NULL          4           ///< Null (none)

/// Debug process definitions
#ifdef RTE_Network_Debug
#define NET_SYSTEM          0           ///< Main system
#define NET_DYN_MEMORY      1           ///< Dynamic memory module
#define NET_ETH_LINK        2           ///< Ethernet interface 0
#define NET_PPP_LINK        3           ///< PPP interface
#define NET_SLIP_LINK       4           ///< SLIP interface
#define NET_LOOP_BACK       5           ///< Loopback interface
#define NET_ARP_CACHE       6           ///< Ethernet ARP cache
#define NET_IP4_CORE        7           ///< IPv4 core and localhost
#define NET_ICMP_CONTROL    8           ///< ICMP control service for IPv4
#define NET_IGMP_HOST       9           ///< IGMP group managememt for IPv4

#define NET_IP6_CORE        10          ///< IPv6 core and localhost
#define NET_ICMP6_CONTROL   11          ///< ICMP6 control service for IPv6
#define NET_NDP_CACHE       12          ///< Ethernet Neighbor Discovery for IPv6
#define NET_UDP_SOCKET      13          ///< UDP native socket
#define NET_TCP_SOCKET      14          ///< TCP native socket
#define NET_BSD_SOCKET      15          ///< BSD socket interface
#define NET_NBNS_CLIENT     16          ///< NBNS client service for IPv4
#define NET_DHCP_CLIENT     17          ///< DHCP client service for IPv4
#define NET_DHCP6_CLIENT    18          ///< DHCP client service for IPv6
#define NET_DNS_CLIENT      19          ///< DNS client service

#define NET_SNMP_AGENT      20          ///< SNMP agent module
#define NET_HTTP_SERVER     21          ///< HTTP server service
#define NET_FTP_SERVER      22          ///< FTP server service
#define NET_FTP_CLIENT      23          ///< FTP client service
#define NET_TELNET_SERVER   24          ///< Telnet server service
#define NET_TFTP_SERVER     25          ///< TFTP server service
#define NET_TFTP_CLIENT     26          ///< TFTP client service
#define NET_SMTP_CLIENT     27          ///< SMTP client service
#define NET_SNTP_CLIENT     28          ///< SNTP client service
#endif /* RTE_Network_Debug */

/// Telnet definitions
#define TELNET_LBUF_SZ      96          ///< Command line buffer size in bytes
#define TELNET_HIST_SZ      128         ///< Command history buffer size in bytes
#define TELNET_FIFO_SZ      128         ///< Input character Fifo buffer in bytes

/// BSD Socket ioctl commands
#define FIONBIO             1           ///< Set mode (blocking/non-blocking)
#define FIO_DELAY_ACK       2           ///< Set DELAY_ACK mode for stream socket
#define FIO_KEEP_ALIVE      3           ///< Set KEEP_ALIVE mode for stream socket
#define FIO_FLOW_CTRL       4           ///< Set FLOW_CTRL mode for stream socket

#ifdef __cplusplus
extern "C"  {
#endif

/// System Error codes
typedef enum net_error {
  NET_ERROR_MEM_ALLOC,                  ///< Alloc memory failed, out of memory
  NET_ERROR_MEM_FREE,                   ///< Free memory failed, memory slot invalid
  NET_ERROR_MEM_CORRUPT,                ///< Memory corruption detected
  NET_ERROR_CONFIG,                     ///< Network configuration error detected
  NET_ERROR_UDP_ALLOC,                  ///< No free UDP sockets available
  NET_ERROR_TCP_ALLOC,                  ///< No free TCP sockets available
  NET_ERROR_TCP_STATE                   ///< TCP socket in undefined state
} NET_ERROR;

/// Frame buffer structure
typedef struct net_frame {
  uint16_t length;                      ///< Length of data in frame
  uint16_t index;                       ///< Buffer position index
  uint8_t  data[1];                     ///< Buffer data (protocol headers + data)
} NET_FRAME;

/// Buffer queue structure
typedef struct net_buffer {
  struct net_buffer *next;              ///< Next buffer in the list
  uint16_t length;                      ///< Length of data in buffer
  uint16_t index;                       ///< Buffer position index
  uint8_t  data[1];                     ///< Buffer data
} NET_BUFFER;

/// Library variant optimization
#ifdef RTE_Network_IPv6
  #define  Network_IPv6
  #define __ADDR            NET_ADDR
  #define __ADDR_IP_LEN     NET_ADDR_IP6_LEN
#else
  #define __ADDR            NET_ADDR4
  #define __ADDR_IP_LEN     NET_ADDR_IP4_LEN
#endif

/// Local Machine info version 4
typedef struct net_localm {
  uint8_t  IpAddr[NET_ADDR_IP4_LEN];    ///< Local IP address
  uint8_t  NetMask[NET_ADDR_IP4_LEN];   ///< Netmask
  uint8_t  DefGW[NET_ADDR_IP4_LEN];     ///< Default gateway
  uint8_t  PriDNS[NET_ADDR_IP4_LEN];    ///< Primary DNS server
  uint8_t  SecDNS[NET_ADDR_IP4_LEN];    ///< Secondary DNS server
} NET_LOCALM;

/// Local Machine info version 6
typedef struct net_localm6 {
  uint16_t Mtu;                         ///< Maximum IP transfer unit
  uint8_t  HopLimit;                    ///< Hop limit for outging IP packets
  uint8_t  PrefixLen;                   ///< Subnet prefix length
  uint8_t  IpAddr[NET_ADDR_IP6_LEN];    ///< IPv6 address
  uint8_t  TempAddr[NET_ADDR_IP6_LEN];  ///< Temporary IPv6 address
  uint8_t  LLAddr[NET_ADDR_IP6_LEN];    ///< Link-local address
  uint8_t  DefGW[NET_ADDR_IP6_LEN];     ///< Default gateway
  uint8_t  PriDNS[NET_ADDR_IP6_LEN];    ///< Primary DNS server
  uint8_t  SecDNS[NET_ADDR_IP6_LEN];    ///< Secondary DNS server
} NET_LOCALM6;

/// ARP Cache Entry info
typedef struct net_arp_info {
  uint8_t  State;                       ///< Cache entry state
  uint8_t  Type;                        ///< Cache entry type
  uint8_t  Retries;                     ///< Number of retries left
  uint8_t  Tout;                        ///< Timeout timer
  uint8_t  IpAddr[NET_ADDR_IP4_LEN];    ///< IPv4 address
  uint8_t  MacAddr[NET_ADDR_ETH_LEN];   ///< Hardware MAC address
} NET_ARP_INFO;

/// IGMP Group info
typedef struct net_igmp_info {
  uint8_t  State;                       ///< Group membership state
  uint8_t  Tout;                        ///< Timeout timer for reports
  uint8_t  Flags;                       ///< State flags
  uint8_t  IpAddr[NET_ADDR_IP4_LEN];    ///< Group IPv4 address
} NET_IGMP_INFO;

/// NDP Cache Entry info
typedef struct net_ndp_info {
  uint8_t  State;                       ///< Cache state
  uint8_t  Flags;                       ///< State flags
  uint16_t Tout;                        ///< Timeout timer
  uint8_t  IpAddr[NET_ADDR_IP6_LEN];    ///< IPv6 address
  uint8_t  MacAddr[NET_ADDR_ETH_LEN];   ///< Hardware MAC address
  uint8_t  Retries;                     ///< Number of retries left
  uint8_t  Reserved;                    ///< Reserved/unused (for alignment)
} NET_NDP_INFO;

/// UDP Socket info
typedef struct net_udp_info {
  uint8_t  State;                       ///< Socket state
  uint8_t  Flags;                       ///< State flags and options
  uint16_t LocPort;                     ///< Local UDP port
  uint8_t  Tos;                         ///< Type of Service for IPv4
  uint8_t  TClass;                      ///< Traffic class/packet priority for IPv6
  uint8_t  McastTtl;                    ///< Multicast time to live for IPv4
  uint8_t  HopLimit;                    ///< Multicast hop limit for IPv6
  netUDP_cb_t cb_func;                  ///< Event callback function
} NET_UDP_INFO;

/// TCP Socket info
typedef struct net_tcp_info {
  netTCP_State State;                   ///< Socket state
  uint8_t  Type;                        ///< Socket type
  uint8_t  Flags;                       ///< State flags
  uint16_t LocPort;                     ///< Local TCP port
  uint8_t  Tos;                         ///< Type of Service for IPv4
  uint8_t  TClass;                      ///< Traffic class/packet priority for IPv6
  const uint8_t *LocAddr;               ///< Pointer to local IPv6 address
  __ADDR   Peer;                        ///< Peer IP address and port
  uint16_t MaxSegSize;                  ///< Transmit max. segment size
  uint16_t ConnTout;                    ///< Connection timeout
  uint16_t AliveTimer;                  ///< Keep alive timer
  uint16_t RetryTimer;                  ///< Retransmission timer
  uint8_t  AckTimer;                    ///< Receive delay-ack timer
  uint8_t  Id;                          ///< Socket identification number
  uint8_t  Retries;                     ///< Number of retries left
  uint8_t  DupAcks;                     ///< Number of duplicate acks (fast recovery)
  uint32_t SendUna;                     ///< Send sequence number unacknowledged
  uint32_t SendNext;                    ///< Next send sequence number
  uint32_t SendChk;                     ///< Check sequence number for dupacks
  uint32_t SendWl1;                     ///< Sequence number of last window update
  uint32_t SendWl2;                     ///< Acknowledge number of last window update
  uint16_t SendWin;                     ///< Current send window
  int16_t  RttSa;                       ///< Scaled average for RTT estimator
  int16_t  RttSv;                       ///< Scaled deviation for RTT estimator
  uint16_t CWnd;                        ///< Congestion window
  uint16_t SsThresh;                    ///< Slow start treshold
  uint16_t RecWin;                      ///< Current receive window
  uint32_t RecNext;                     ///< Next receive sequence number
  NET_FRAME *unack_list;                ///< Unacked queue list
  netTCP_cb_t cb_func;                  ///< Event callback function
} NET_TCP_INFO;

/// BSD Socket info
typedef struct net_bsd_info {
  uint8_t  State;                       ///< Socket state
  uint8_t  Socket;                      ///< Assigned TCP/UDP socket
  uint8_t  Flags;                       ///< Flags for socket option
  uint8_t  Type;                        ///< Type of socket
  __ADDR   RemHost;                     ///< Remote IP address and port
  __ADDR   LocHost;                     ///< Local IP address and port
  uint8_t  AcceptSock;                  ///< Accept socket list
  uint8_t  ParentSock;                  ///< Parent (server) socket
  uint16_t RcvTout;                     ///< Blocking receive timeout
  uint16_t Timer;                       ///< Receive or transmit timer
  uint8_t  Event;                       ///< Owner wait-for events
  uint8_t  xEvent;                      ///< Sender wait-for events
  uint8_t  Tos;                         ///< Type of Service for IPv4
  uint8_t  TClass;                      ///< Traffic class for IPv6
  uint8_t  Id;                          ///< Socket identification number
  void    *Thread;                      ///< Socket owner thread
  void    *xThread;                     ///< Socket sender thread
  void    *buf_list;                    ///< Rx data buffer list
} NET_BSD_INFO;

/// DNS Cache Entry info
typedef struct net_dns_info {
  uint32_t HostId;                      ///< Host Id (CRC32 value of host name)
  uint16_t Tout;                        ///< Cache timeout timer
  uint16_t Type;                        ///< Address type: IPv4 or IPv6
  uint8_t  IpAddr[__ADDR_IP_LEN];       ///< Internet IP address
} NET_DNS_INFO;

/// TFTP Server Session info
typedef struct net_tftp_info {
  uint8_t  State;                       ///< Session state
  uint8_t  Socket;                      ///< Assigned UDP socket
  uint8_t  Flags;                       ///< State flags
  uint8_t  Retries;                     ///< Retry counter
  __ADDR   Client;                      ///< Client IP address and port (TID)
  uint16_t BlockSz;                     ///< Transfer block size
  uint16_t BlockNr;                     ///< Block number
  uint8_t  Timer;                       ///< Timeout timer
  uint16_t BufLen;                      ///< Length of retransmit buffer
  uint8_t  *Buf;                        ///< Transmit/retransmit buffer
  void     *File;                       ///< File handle pointer
} NET_TFTP_INFO;

/// Telnet Session info
typedef struct net_telnet_info {
  uint8_t  State;                       ///< Session state
  uint8_t  Socket;                      ///< Assigned TCP socket
  uint8_t  Flags;                       ///< State flags
  uint8_t  BCnt;                        ///< Received data byte count
  uint16_t Tout;                        ///< Timeout delay timer
  uint8_t  UserId;                      ///< User Id (authentication enabled)
  uint8_t  Widx;                        ///< Fifo buffer write index
  uint8_t  Ridx;                        ///< Fifo buffer read index
  uint8_t  hNext;                       ///< History command next position
  uint8_t  hCurr;                       ///< History command current position
  uint32_t SVar;                        ///< Session private variable
  char     LBuf[TELNET_LBUF_SZ];        ///< Data line buffer
  char     Fifo[TELNET_FIFO_SZ];        ///< Fifo buffer for received line/keycodes
  char     Hist[TELNET_HIST_SZ];        ///< Command history buffer
} NET_TELNET_INFO;

/// FTP Session info
typedef struct net_ftp_info {
  uint8_t  State;                       ///< Session state
  uint8_t  Socket;                      ///< TCP control socket
  uint16_t Flags;                       ///< State flags
  __ADDR   Client;                      ///< Client IP address and port
  uint16_t DPort;                       ///< Data port (active or passive)
  uint8_t  DSocket;                     ///< TCP data socket
  uint8_t  UserId;                      ///< User Id (authentication enabled)
  uint8_t  Resp;                        ///< Server response code
  uint8_t  PathLen;                     ///< Size of the path string
  char    *Path;                        ///< Current working directory
  char    *Name;                        ///< Absolute file/folder path name
  void    *File;                        ///< File handle pointer
} NET_FTP_INFO;

/// HTTP Session info
typedef struct net_http_info {
  uint8_t  State;                       ///< Session state
  uint8_t  Socket;                      ///< Assigned TCP socket
  uint16_t Flags;                       ///< State flags
  uint8_t  FType;                       ///< File type: html, jpeg, gif,...
  uint8_t  PostSt;                      ///< POST method processing state
  uint16_t DelimSz;                     ///< Boundary delimiter size
  uint8_t  UserId;                      ///< User Id (authentication enabled)
  uint32_t CGIvar;                      ///< CGI session private variable
  uint32_t DLen;                        ///< Data length (number of bytes to send)
  uint32_t Count;                       ///< Total number of bytes sent
  uint16_t BCnt;                        ///< Number of bytes sent in last packet
  char     Lang[6];                     ///< Selected web language: en-us, de, fr...
  uint32_t LMDate;                      ///< Last modified date (UTC format)
  char    *Script;                      ///< Script buffer pointer
  char    *pDelim;                      ///< Multipart boundary delimiter value
  char    *pUser;                       ///< User MIME type or Redirect URL
  void    *sFile;                       ///< Script interpreter file pointer
  void    *dFile;                       ///< Data file pointer
} NET_HTTP_INFO;

/// HTTP Error page info
typedef struct net_http_error {
  const char *header;                   ///< Common page header
  const char *footer;                   ///< Common page footer
  const char *e401;                     ///< 401 Unauthorized
  const char *e403;                     ///< 403 Forbidden
  const char *e404;                     ///< 404 Not Found
  const char *e501;                     ///< 501 Not Implemented
} const NET_HTTP_ERROR;

/// SYS Configuration info
typedef struct net_sys_cfg {
  uint32_t   *MemPool;                  ///< Dynamic memory pool buffer
  uint32_t   MemSize;                   ///< Memory pool size in bytes
  const void *ThreadDef;                ///< Network-Core thread definition
  const void *MutexDef;                 ///< Network-Core mutex definition
  const void *MemMutexDef;              ///< Memory manager mutex definition
  char       *Path;                     ///< Path buffer for root-enabled services
  const char *HostName;                 ///< Configured Local host name
  bool       AutoStart;                 ///< Auto-Start server service
  uint8_t    NetCfg[3];                 ///< Network interface configuration flags
} const NET_SYS_CFG;

/// ARP Configuration info
typedef struct net_arp_cfg {
  NET_ARP_INFO *Table;                  ///< Cache table array
  uint8_t  TabSize;                     ///< Cache table size
  uint8_t  TimeOut;                     ///< Cache expiration time in seconds
  uint8_t  MaxRetry;                    ///< Number of retries to resolve MAC address
  uint8_t  Resend;                      ///< Resend timeout in seconds
  bool     Notify;                      ///< Notify on IP address changes
} const NET_ARP_CFG;

/// IGMP Configuration info
typedef struct net_igmp_cfg {
  NET_IGMP_INFO *Table;                 ///< Group table array
  uint16_t TabSize;                     ///< Group table size
} const NET_IGMP_CFG;

/// DHCP Client Configuration info
typedef struct net_dhcp_cfg {
  const char *Vcid;                     ///< Vendor class identifier
  uint8_t    Opt;                       ///< Extra DHCP option flags
} const NET_DHCP_CFG;

/// IP4 Configuration info
typedef struct net_ip4_cfg {
  const char  *IpAddr;                  ///< IPv4 address
  const char  *NetMask;                 ///< IPv4 network mask
  const char  *DefGW;                   ///< Default gateway IPv4 address
  const char  *PriDNS;                  ///< Primary DNS server
  const char  *SecDNS;                  ///< Secondary DNS server
  NET_ARP_CFG  ArpCfg;                  ///< ARP configuration
  NET_IGMP_CFG IgmpCfg;                 ///< IGMP configuration
  NET_DHCP_CFG DhcpCfg;                 ///< DHCP configuraton
} const NET_IP4_CFG;

/// NDP Configuration info
typedef struct net_ndp_cfg {
  NET_NDP_INFO *Table;                  ///< Neighbor cache table
  uint8_t   TabSize;                    ///< Cache table size
  uint8_t   TimeOut;                    ///< Cache expiration time in seconds
  uint8_t   MaxRetry;                   ///< Number of retries to resolve MAC address
  uint8_t   Resend;                     ///< Resend timeout in seconds
} const NET_NDP_CFG;

/// DHCP6 Client Configuration info
typedef struct net_dhcp6_cfg {
  uint8_t    Mode;                      ///< DHCPv6 mode (stateless/stateful)
  uint8_t    Opt;                       ///< Extra DHCPv6 option flags
  uint32_t   EnterpId;                  ///< Enterprise-number
  const char *VcData;                   ///< Vendor class data
} const NET_DHCP6_CFG;

/// IP6 Configuration info
typedef struct net_ip6_cfg {
  const char   *IpAddr;                 ///< IPv6 address
  const char   *DefGW;                  ///< Default gateway IPv6 address
  const char   *PriDNS;                 ///< Primary DNS server
  const char   *SecDNS;                 ///< Secondary DNS server
  uint8_t       PrefixLen;              ///< Address prefix length
  NET_NDP_CFG   NdpCfg;                 ///< Neighbor discovery configuration
  NET_DHCP6_CFG DhcpCfg;                ///< DHCPv6 configuration
} const NET_IP6_CFG;

/// Ethernet Interface Configuration info
typedef struct net_eth_cfg {
  ARM_DRIVER_ETH_MAC *DrvMac;           ///< Registered MAC driver
  ARM_DRIVER_ETH_PHY *DrvPhy;           ///< Registered PHY driver
  const void  *ThreadDef;               ///< Thread definition
  const void  *SemaphoreDef;            ///< Semaphore definition
  uint8_t     *MacAddr;                 ///< Active MAC address
  const char  *MacCfg;                  ///< Configured MAC address
  NET_IP4_CFG *Ip4Cfg;                  ///< IPv4 configuration
  NET_IP6_CFG *Ip6Cfg;                  ///< IPv6 configuration
} const NET_ETH_CFG;

/// Serial Device Configuration info
typedef struct net_com_cfg {
  ARM_DRIVER_USART *DrvUsart;           ///< Registered USART driver
  DRIVER_MODEM     *DrvModem;           ///< Registered MODEM driver
  const void  *ThreadDef;               ///< Thread definition
  const void  *SemaphoreDef;            ///< Semaphore definition
  const char  *InitString;              ///< Modem initialization string
  uint32_t    Speed;                    ///< Connection speed
  uint8_t     FlowCtrl;                 ///< Flow control
  uint8_t     MaxDelay;                 ///< Max. thread polling delay in ms
  uint8_t     RecTout;                  ///< Frame receive timeout in ticks
} const NET_COM_CFG;

/// PPP Interface Configuration info
typedef struct net_ppp_cfg {
  NET_COM_CFG ComDev;                   ///< Serial device configuration
  uint32_t    ACCmap;                   ///< Async control character map
  uint16_t    RetryTout;                ///< Retry timeout in ticks
  uint16_t    EchoTout;                 ///< LCP echo timeout in seconds
  uint8_t     MaxRetry;                 ///< Number of retries
  uint8_t     EnAuth;                   ///< Enabled authentication protocols
  const char *IpAddr;                   ///< IPv4 address
  const char *PriDNS;                   ///< Primary DNS server
  const char *SecDNS;                   ///< Secondary DNS server
} const NET_PPP_CFG;

/// SLIP Interface Configuration info
typedef struct net_slip_cfg {
  NET_COM_CFG ComDev;                   ///< Serial device configuration
  const char *IpAddr;                   ///< IPv4 address
  const char *PriDNS;                   ///< Primary DNS server
  const char *SecDNS;                   ///< Secondary DNS server
} const NET_SLIP_CFG;

/// UDP Socket Configuration info
typedef struct net_udp_cfg {
  NET_UDP_INFO *Scb;                    ///< Socket Control Block array
  uint8_t  NumSocks;                    ///< Number of UDP Sockets
} const NET_UDP_CFG;

/// TCP Socket Configuration info
typedef struct net_tcp_cfg {
  NET_TCP_INFO *Scb;                    ///< Socket control block array
  uint8_t  NumSocks;                    ///< Number of TCP sockets
  uint8_t  MaxRetry;                    ///< Number of retries
  uint16_t RetryTout;                   ///< Retry timeout in ticks
  uint16_t SynRetryTout;                ///< SYN retry timeout in ticks
  uint16_t InitRetryTout;               ///< Initial retransmit timeout in ticks
  uint16_t DefTout;                     ///< Default connect timeout in seconds
  uint16_t MaxSegSize;                  ///< Maximum segment size value
  uint16_t RecWinSize;                  ///< Receiving window size in bytes
  uint8_t  ConnRetry;                   ///< Number of retries to connect
} const NET_TCP_CFG;

/// BSD Socket Configuration info
typedef struct net_bsd_cfg {
  NET_BSD_INFO *Scb;                    ///< Socket control block array
  uint8_t  NumSocks;                    ///< Number of BSD sockets
  uint16_t RcvTout;                     ///< Blocking recv timeout in ticks
} const NET_BSD_CFG;

/// DNS Client Configuration info
typedef struct net_dns_cfg {
  NET_DNS_INFO *Table;                  ///< Cache table array
  uint8_t   TabSize;                    ///< Cache table size
} const NET_DNS_CFG;

/// SNMP Agent Configuration info
typedef struct net_snmp_cfg {
  uint16_t   PortNum;                   ///< Listening port number
  uint16_t   TrapPort;                  ///< Trap port number
  const char *TrapIp;                   ///< Trap server IP address
  const char *Community;                ///< Community name
} const NET_SNMP_CFG;

/// SNTP Client Configuration info
typedef struct net_sntp_cfg {
  const char *SrvIp;                    ///< NTP/SNTP server IP address
  bool       Bcast;                     ///< Broadcast client mode (for LAN only)
} const NET_SNTP_CFG;

/// TFTP Server Configuration info
typedef struct net_tftp_cfg {
  NET_TFTP_INFO *Scb;                   ///< Session control block array
  uint8_t  NumSess;                     ///< Max. number of active sessions
  uint8_t  MaxRetry;                    ///< Number of retries
  uint16_t PortNum;                     ///< Listening port number
  uint8_t  DefTout;                     ///< Inactive session timeout in seconds
  bool     EnFwall;                     ///< Enable firewall support
  const char *Root;                     ///< Root folder path
} const NET_TFTP_CFG;

/// TFTP Client Configuration info
typedef struct net_tftpc_cfg {
  uint16_t BlockSize;                   ///< Transfer block size
  uint16_t RetryTout;                   ///< Retry timeout in ticks
  uint8_t  MaxRetry;                    ///< Number of retries
} const NET_TFTPC_CFG;

/// Telnet Server Configuration info
typedef struct net_telnet_cfg {
  NET_TELNET_INFO *Scb;                 ///< Session control block array
  uint8_t  NumSess;                     ///< Max. number of active sessions
  bool     EnAuth;                      ///< Enable user authentication
  uint8_t  NoEcho;                      ///< Disable server echo mode
  uint16_t PortNum;                     ///< Listening port number
  uint16_t IdleTout;                    ///< Idle connection timeout in ticks
  const char *UserCfg;                  ///< Default admin user name
  const char *PasswCfg;                 ///< Default admin password
  char    *User;                        ///< Current admin user name
  char    *Passw;                       ///< Current admin password
} const NET_TELNET_CFG;

/// FTP Server Configuration info
typedef struct net_ftp_cfg {
  NET_FTP_INFO *Scb;                    ///< Session control block array
  uint8_t  NumSess;                     ///< Max. number of active sessions
  bool     EnAuth;                      ///< Enable user authentication
  uint16_t PortNum;                     ///< Listening port number
  uint16_t IdleTout;                    ///< Idle connection timeout in ticks
  uint8_t  MsgLen;                      ///< Length of welcome message
  const char *Msg;                      ///< Server welcome message
  const char *Root;                     ///< Root folder path
  const char *UserCfg;                  ///< Default admin user name
  const char *PasswCfg;                 ///< Default admin password
  char    *User;                        ///< Current admin user name
  char    *Passw;                       ///< Current admin password
} const NET_FTP_CFG;

/// FTP Client Configuration info
typedef struct net_ftpc_cfg {
  uint8_t DefTout;                      ///< Default inactivity timeout
  bool    PasvMode;                     ///< FTP passive mode used
} const NET_FTPC_CFG;

/// HTTP Server Configuration info
typedef struct net_http_cfg {
  NET_HTTP_INFO *Scb;                   ///< Session control block array
  uint8_t   NumSess;                    ///< Max. number of active sessions
  bool      EnAuth;                     ///< Enable user authentication
  bool      EnTLS;                      ///< Enable HTTPS secure mode
  uint16_t  PortNum;                    ///< Listening port number
  const char *SrvId;                    ///< Server-Id http header
  const char *Root;                     ///< Root folder path
  const char *Realm;                    ///< Authentication realm space
  const char *UserCfg;                  ///< Default admin user name
  const char *PasswCfg;                 ///< Default admin password
  char     *User;                       ///< Current admin user name
  char     *Passw;                      ///< Current admin password
} const NET_HTTP_CFG;

/// Network scheduler function type
typedef void (*net_sys_fn_t) (void);

/// Network scheduler function table
extern const net_sys_fn_t net_sys_func[][2];
#define NET_SYS_FUNC_END ((net_sys_fn_t)0xFFFFFFFF)

/// Generic NULL address
extern const uint8_t net_addr_null[16];

//  ==== Network system ===

/// \brief Network Component core thread.
/// \param[in]     arg           dummy parameter.
/// \return        None.
extern void netCore_Thread (void const *arg);

/// \brief Signal Network Component error.
/// \param[in]     error         System error code.
/// \return        None.
extern void net_sys_error (NET_ERROR error);

/// \brief Acquire Network core protection mutex.
/// \return        None.
extern void net_sys_lock   (void);

/// \brief Release Network core protection mutex.
/// \return        None.
extern void net_sys_unlock (void);

//  ==== Memory management ===

/// \brief Allocate memory for the network frame.
/// \param[in]     byte_size     buffer size in bytes.
/// \return        Pointer to the allocated memory.
extern NET_FRAME *net_mem_alloc (uint32_t byte_size);

/// \brief Release allocated memory.
/// \param[in]     mem_ptr       pointer to the allocated memory.
/// \return        None.
extern void net_mem_free (NET_FRAME *mem_ptr);

//  ==== Ethernet interface ===

/// \brief Ethernet interface thread.
/// \param[in]     arg           dummy parameter.
/// \return        None.
extern void netETH_Thread (void const *arg);

/// \brief Initialize Ethernet interface.
/// \return        None.
extern void net_eth_iface_init (void);

/// \brief Run ethernet interface main function.
/// \return None.
extern void net_eth_iface_run (void);

/// \brief Construct ethernet header for the frame and send it.
/// \param[in]     frame         frame to be sent.
/// \param[in]     ip_ver        IP version of the frame.
/// \return
///              - true        = Frame successfuly sent.
///              - false       = Failed to send a frame.
extern bool net_eth_send_frame (NET_FRAME *frame, uint8_t ip_ver);

/// \brief Check if ethernet destination MAC address is our local MAC address.
/// \param[in]     frame         received frame to be checked.
/// \return
///              - true        = Destination MAC is local MAC address.
///              - false       = Destination MAC is not local.
extern bool net_eth_is_my_addr (const NET_FRAME *frame);

/// \brief Get MAC address for the given IP address.
/// \param[in]     addr        structure containing IP address of a host.
/// \return        Pointer to the MAC address in Neighbor cache.
extern const uint8_t *net_eth_get_addr (const __ADDR *addr);

/// \brief Get current value of Ethernet Interface option.
/// \param[in]     if_num        Interface number.
/// \param[in]     option        Interface option as specified by \ref netIF_Option.
/// \param[out]    buf           buffer to store the option value to
/// \param[in]     buf_len       length of buffer
/// \return      status code that indicates the execution status of the function as defined with \ref netStatus.
extern netStatus net_eth_get_option (uint32_t if_num, netIF_Option option, uint8_t *buf, uint32_t buf_len);

/// \brief Set the value of Ethernet Interface option.
/// \param[in]     if_num        Interface number.
/// \param[in]     option        Interface option as specified by \ref netIF_Option.
/// \param[in]     buf           buffer containing the option value
/// \param[in]     buf_len       length of buffer
/// \return      status code that indicates the execution status of the function as defined with \ref netStatus.
extern netStatus net_eth_set_option (uint32_t if_num, netIF_Option option, const uint8_t *buf, uint32_t buf_len);

/// \brief Send a notification of local IP address change (gratuitous ARP).
/// \return        None.
extern void net_arp_notify (void);

/// \brief Initialize IGMP group management.
/// \return        None.
extern void net_igmp_host_init (void);

/// \brief Run IGMP group management main function.
/// \return        None.
extern void net_igmp_host_run (void);

/// \brief Process IGMP group management frame.
/// \param[in]     frame         received IGMP group management frame.
/// \return        None.
extern void net_igmp_process (NET_FRAME *frame);

/// \brief Check if local host is a member of provided group.
/// \param[in]     ip4_addr      multicast IPv4 address to be checked.
/// \return
///              - true        = Local host is member of IGMP group ipadr.
///              - false       = Local host is not member.
extern bool net_igmp_is_member (const uint8_t *ip4_addr);

/// \brief Collect IP addresses of active IGMP groups.
/// \param[out]    buf         buffer to copy group IP addresses to.
/// \return        Number of IPs copied.
extern int32_t net_igmp_collect_addr (uint8_t *buf);

/// \brief Process Neighbor Discovery frame.
/// \param[in]     frame         received ND frame.
/// \return        None.
extern void net_ndp_process (NET_FRAME *frame);

/// \brief Initialize NBNS client.
/// \return        None.
extern void net_nbns_client_init (void);

/// \brief Initialize DHCP client.
/// \return        None.
extern void net_dhcp_client_init (void);

/// \brief Run DHCP client main function.
/// \return        None.
extern void net_dhcp_client_run (void);

/// \brief Initialize DHCP6 client.
/// \return        None.
extern void net_dhcp6_client_init (void);

/// \brief Run DHCP6 client main function.
/// \return        None.
extern void net_dhcp6_client_run (void);

//  ==== PPP interface ===

/// \brief PPP interface thread.
/// \param[in]     arg           dummy parameter.
/// \return        None.
extern void netPPP_Thread (void const *arg);

/// \brief Initialize PPP interface.
/// \return        None.
extern void net_ppp_iface_init (void);

/// \brief Run PPP interface main function.
/// \return        None.
extern void net_ppp_iface_run (void);

/// \brief Construct PPP header for the frame and send it.
/// \param[in]     frame         frame to be sent.
/// \param[in]     prot          PPP protocol type of the frame.
/// \return
///              - true        = Frame successfuly sent.
///              - false       = Failed to send a frame.
extern bool net_ppp_send_frame (NET_FRAME *frame, uint16_t prot);

/// \brief Get current value of PPP Interface option.
/// \param[in]     option        Interface option as specified by \ref netIF_Option.
/// \param[out]    buf           buffer to store the option value to
/// \param[in]     buf_len       length of buffer
/// \return      status code that indicates the execution status of the function as defined with \ref netStatus.
extern netStatus net_ppp_get_option (netIF_Option option, uint8_t *buf, uint32_t buf_len);

/// \brief Set the value of PPP Interface option.
/// \param[in]     option        Interface option as specified by \ref netIF_Option.
/// \param[in]     buf           buffer containing the option value
/// \param[in]     buf_len       length of buffer
/// \return      status code that indicates the execution status of the function as defined with \ref netStatus.
extern netStatus net_ppp_set_option (netIF_Option option, const uint8_t *buf, uint32_t buf_len);

/// \brief Initialize PAP authentication.
/// \return        None.
extern void net_pap_init (void);

/// \brief Run PAP authentication main function.
/// \return        None.
extern void net_pap_run (void);

/// \brief Process PAP authentication frame.
/// \param[in]     frame         received PAP authentication frame.
/// \return        None.
extern void net_pap_process (NET_FRAME *frame);

/// \brief Initialize CHAP authentication.
/// \return        None.
extern void net_chap_init (void);

/// \brief Run CHAP authentication main function.
/// \return        None.
extern void net_chap_run (void);

/// \brief Process CHAP authentication frame.
/// \param[in]     frame         received CHAP authentication frame.
/// \return        None.
extern void net_chap_process (NET_FRAME *frame);

//  ==== SLIP interface ===

/// \brief SLIP interface thread.
/// \param[in]     arg           dummy parameter.
/// \return        None.
extern void netSLIP_Thread (void const *arg);

/// \brief Initialize SLIP interface.
/// \return        None.
extern void net_slip_iface_init (void);

/// \brief Run SLIP interface main function.
/// \return        None.
extern void net_slip_iface_run (void);

/// \brief Send a SLIP frame.
/// \param[in]     frame         frame to be sent.
/// \param[in]     ip_ver        IP version of the frame.
/// \return
///              - true        = Frame successfuly sent.
///              - false       = Failed to send a frame.
extern bool net_slip_send_frame (NET_FRAME *frame, uint8_t ip_ver);

/// \brief Get current value of SLIP Interface option.
/// \param[in]     option        Interface option as specified by \ref netIF_Option.
/// \param[out]    buf           buffer to store the option value to
/// \param[in]     buf_len       length of buffer
/// \return      status code that indicates the execution status of the function as defined with \ref netStatus.
extern netStatus net_slip_get_option (netIF_Option option, uint8_t *buf, uint32_t buf_len);

/// \brief Set the value of PPP Interface option.
/// \param[in]     option        Interface option as specified by \ref netIF_Option.
/// \param[in]     buf           buffer containing the option value
/// \param[in]     buf_len       length of buffer
/// \return      status code that indicates the execution status of the function as defined with \ref netStatus.
extern netStatus net_slip_set_option (netIF_Option option, const uint8_t *buf, uint32_t buf_len);

//  ==== Network core ===

/// \brief Initialize loopback interface.
/// \return        None.
extern void net_loop_iface_init (void);

/// \brief Run loopback interface main function.
/// \return        None.
extern void net_loop_iface_run (void);

/// \brief Initialize Ping client.
/// \return        None.
extern void net_ping_client_init (void);

/// \brief Run Ping client main function.
/// \return        None.
extern void net_ping_client_run (void);

/// \brief Initialize UDP sockets.
/// \return        None.
extern void net_udp_socket_init (void);

/// \brief Process UDP frame.
/// \param[in]     frame         received UDP frame.
/// \param[in]     ip_ver        received IP version.
/// \return        None.
extern void net_udp_process (NET_FRAME *frame, uint8_t ip_ver);

/// \brief Initialize TCP sockets.
/// \return        None.
extern void net_tcp_socket_init (void);

/// \brief Run TCP socket main function.
/// \return        None.
extern void net_tcp_socket_run (void);

/// \brief Process TCP frame.
/// \param[in]     frame         received TCP frame.
/// \param[in]     ip_ver        received IP version.
/// \return        None.
extern void net_tcp_process (NET_FRAME *frame, uint8_t ip_ver);

/// \brief Allocate a free TCP socket.
/// \param[in]     cb_func       event listening callback function.
/// \return      socket handle number or execution status:
///              - value >= 0:   socket handle number
///              - value < 0:    error occurred, -value is execution status as defined with \ref netStatus 
extern int32_t net_tcp_get_socket (netTCP_cb_t cb_func);

/// \brief Convert TCP socket state into a string.
/// \param[in]     state         socket state as defined with netTCP_State.
/// \return        pointer to the text representation of a socket state.
extern const char *net_tcp_ntoa (netTCP_State state);

/// \brief Initialize DNS client.
/// \return        None.
extern void net_dns_client_init (void);

/// \brief Run DNS client main function.
/// \return        None.
extern void net_dns_client_run (void);

/// \brief Initialize BSD sockets.
/// \return        None.
extern void net_bsd_socket_init (void);

/// \brief Run BSD socket main function.
/// \return        None.
extern void net_bsd_socket_run (void);

/// \brief Initialize BSD host resolver.
/// \return        None.
extern void net_bsd_host_init (void);

//  ==== FTP server service ===

/// \brief Initialize FTP server.
/// \return        None.
extern void net_ftp_server_init (void);

/// \brief Run FTP server main function.
/// \return        None.
extern void net_ftp_server_run (void);

//  ==== FTP client application ===

/// \brief Initialize FTP client.
/// \return        None.
extern void net_ftp_client_init (void);

/// \brief Run FTP client main function.
/// \return        None.
extern void net_ftp_client_run (void);

//  ==== TFTP server service ===

/// \brief Initialize TFTP server.
/// \return        None.
extern void net_tftp_server_init (void);

/// \brief Run TFTP server main function.
/// \return        None.
extern void net_tftp_server_run (void);

//  ==== TFTP client application ===

/// \brief Initialize TFTP client.
/// \return        None.
extern void net_tftp_client_init (void);

/// \brief Run TFTP client main function.
/// \return        None.
extern void net_tftp_client_run (void);

//  ==== SMTP client application ===

/// \brief Initialize SMTP client.
/// \return        None.
extern void net_smtp_client_init (void);

/// \brief Run SMTP client main function.
/// \return        None.
extern void net_smtp_client_run (void);

//  ==== SNMP agent application ===

/// \brief Initialize SNMP agent.
/// \return        None.
extern void net_snmp_agent_init (void);

/// \brief Run SNMP agent main function.
/// \return        None.
extern void net_snmp_agent_run (void);

//  ==== SNTP client application ===

/// \brief Initialize SNTP client.
/// \return        None.
extern void net_sntp_client_init (void);

/// \brief Run SNTP client main function.
/// \return        None.
extern void net_sntp_client_run (void);

//  ==== Telnet server service ===

/// \brief Initialize Telnet server.
/// \return        None.
extern void net_telnet_server_init (void);

/// \brief Run Telnet server main function.
/// \return        None.
extern void net_telnet_server_run (void);

//  ==== HTTP server service ===

/// \brief Initialize HTTP server.
/// \return        None.
extern void net_http_server_init (void);

/// \brief Run HTTP server main function.
/// \return        None.
extern void net_http_server_run (void);

/// \brief Convert generic time to internal HTTP time format.
/// \param[in]     hr            hours   [0..23].
/// \param[in]     min           minutes [0..59].
/// \param[in]     sec           seconds [0..59].
/// \param[in]     day           day     [1..31].
/// \param[in]     mon           month   [1..12].
/// \param[in]     year          year    [1980..2107].
/// \return        converted time in UTC format.
extern uint32_t net_http_time (uint8_t hr, uint8_t min, uint8_t sec, 
                               uint8_t day, uint8_t mon, uint16_t year);

//  ==== TLS interface ===

/// \brief Initialize TLS interface.
/// \return        None.
extern void netTLS_init (void);

/// \brief Allocate TLS socket and assign a TCP socket.
/// \param[in]     cb_func       event listening TCP callback function.
/// \return        TCP socket handle number or 0= no socket assigned.
extern int32_t netTLS_socket (netTCP_cb_t cb_func);

/// \brief Allocate memory for TLS send buffer.
/// \param[in]     size          number of bytes to allocate.
/// \return        pointer to the allocated memory.
extern uint8_t *netTLS_buf (uint32_t size);

/// \brief Stop TLS communication and close socket.
/// \param[in]     sock          socket descriptor.
/// \return        None.
extern void netTLS_close (int32_t sock);

///  \brief Send data on connected TLS socket.
///  \param[in]    sock          socket descriptor.
///  \param[in]    buf           pointer to data buffer.
///  \param[in]    len           length of data (in bytes).
///  \return       None.
extern void netTLS_send (int32_t sock, const uint8_t *buf, uint32_t len);

//  ==== Network address conversion ===

/// \brief Copy IP address and port.
/// \param[out]    dst_addr      destination IPv4 or IPv6 address.
/// \param[in]     src_addr      source IPv4 or IPv6 address.
/// \return        None.
extern void net_addr_copy (__ADDR *dst_addr, const __ADDR *src_addr);

/// \brief Convert IPv4 address from binary to string.
/// \param[in]     ip4_addr      IPv4 address in binary form.
/// \return        pointer to converted string.
extern const char *net_addr4_ntoa (const uint8_t *ip4_addr);

/// \brief Convert IPv4 address from text to binary form.
/// \param[in]     cp            IPv4 address string (ie. "192.168.1.100").
/// \param[out]    ip4_addr      4-byte address in binary form.
/// \return
///                - true  = successfull,
///                - false = failed.
extern bool net_addr4_aton (const char *cp, uint8_t *ip4_addr);

/// \brief Convert MAC address from binary to string.
/// \param[in]     mac_addr      MAC address in binary.
/// \return        pointer to converted string.
extern const char *net_mac_ntoa (const uint8_t *mac_addr);

#ifdef __cplusplus
}
#endif

#endif /* __RL_NET_LIB_H */
